<?php
// $Id: assistant_api.form.inc,v 1.2 2010/03/10 16:22:50 pounard Exp $

/**
 * @file
 * Solace API UI filter form.
 */

/**
 * Real edit filter form implementation.
 * @see assistant_api_filters_form()
 */
function assistant_api_filters_form($form_state, AbstractAssistantContext $context = NULL) {
  // Ensure loading all needed classes.
  assistant_api_filters_boostrap();
  drupal_add_css(drupal_get_path('module', 'assistant_api') . '/assistant_api.ui.css');
  drupal_add_js('misc/autocomplete.js');

  $form = array(
    '#tree' => TRUE,
  );

  if (! $context) {
    $context = assistant_api_context_create();
  }

  $do_preview = FALSE;

  // Store the context for validation and submit functions
  // TODO: this is ugly, should find a better way to do this
  $form['context'] = array(
    '#type' => 'value',
    '#value' => $context,
  );
  // This one will allow us to bootstrap the module specific classes.
  if (($module = $context->getModule()) != 'global') {
    $form['context_module'] = array(
      '#type' => 'value',
      '#value' => $module,
    );
  }

  // Register form for ahah processing through ahah_helper module
  ahah_helper_register($form, $form_state);
  // Workaround for our weird ahah helper usage
  $form_state['storage']['#ahah_helper']['file'] = drupal_get_path('module', 'assistant_api') . '/assistant_api.form.inc';

  // Display token module help if module is enabled.
  if (module_exists('token')) {
    $form['token_help'] = array(
      '#type' => 'fieldset',
      '#title' => t('Tokens help'),
      '#collapsible' => FALSE,
    );
    $form['token_help']['general'] = array(
      '#type' => 'markup',
      '#prefix' => '<div class="help">',
      '#suffix' => '</div>',
      '#value' => t("In order to do more advanced queries, you can use tokens in any textfield provided by filters. See available tokens below."),
    );
    $form['token_help']['custom'] = array(
      '#type' => 'markup',
      '#prefix' => '<div class="help">',
      '#suffix' => '</div>',
      '#value' => $context->getTokenHelp(),
    );
    $form['token_help']['advanced'] = array(
      '#type' => 'fieldset',
      '#title' => t('See all available tokens'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['token_help']['advanced']['help'] = array(
      '#type' => 'markup',
      '#prefix' => '<div class="help">',
      '#suffix' => '</div>',
      '#value' => theme('token_help', $context->getAllowedTokenTypes()),
    );
  }

  // Ahah container
  $form['data'] = array(
    '#type' => 'markup',
    '#prefix' => '<div id="filters-data-wrapper">',
    '#suffix' => '</div>',
    '#value' => '<span> </span>', // Foo text, so the div appears anyway
    '#theme' => 'assistant_api_filters_form_data',
  );

  // ahah_helper processing, add or remove filter
  if (isset($form_state['storage']['new'])) {
    $add = FALSE;
    $storage = &$form_state['storage']['data'];

    // Check for clicked button
    if (preg_match('/^remove-([0-9]+)$/', $form_state['clicked_button']['#name'], $matches)) {
      unset($storage[(int) $matches[1]]);
    }
    else if ($form_state['clicked_button']['#name'] == 'add_filter') {
      $add = TRUE;
    }
    else if ($form_state['clicked_button']['#name'] == 'live_preview') {
      $do_preview = TRUE;
    }

    $count = 0;

    // Avoid some PHP warnings
    if (empty($storage)) {
      $storage = array();
    }
    else {
      // Then we can rebuild the whole ahah part
      foreach ($storage as $delta => &$data) {
        $name = array_shift(array_keys($data));
        // $delta is important here, it avoids a decal with remove button
        $form['data'][$delta][$name] = _assistant_api_filters_form($context, $name, $delta, $storage[$delta][$name]);
        $count++;
      }
    }

    // Set new filter type in custom storage, if we not removing one
    if ($add && ($name = $form_state['storage']['new']['filter_name'])) {
      $values = array();
      $form['data'][$count][$name] = _assistant_api_filters_form($context, $name, $count, $values);
    }
  }

  // Load filters if no AJAX submission processed
  else if ($fid = $context->getFid()) {
    $filters = assistant_api_filters_load($fid);
    $form['data'] += _assistant_api_filters_subform($context, $filters);
  }

  foreach ($form['data'] as $delta => &$subform) {
    if (is_array($subform)) {
      // Remove button
      $button_delta = 'remove-' . $delta;
      $subform['remove'] = array(
        '#type' => 'button',
        '#value' => t("Remove"),
        '#ahah' => array(
          'event' => 'click',
          'path' => ahah_helper_path(array('data')),
          'wrapper' => 'filters-data-wrapper',
          'effect' => 'none',
          'method' => 'replace',
        ),
        '#name' => $button_delta,
        '#submit' => array(), // Important, this skips normal submit
      );
    }
  }

  $form['new'] = _assistant_api_filters_add_subform($context);
  $form['new']['#prefix'] = '<div class="container-inline">';
  $form['new']['#suffix'] = '</div>';
  $form['new']['add'] = array(
    '#type' => 'button',
    '#value' => t("Add"),
    '#ahah' => array(
      'event' => 'click',
      'path' => ahah_helper_path(array('data')),
      'wrapper' => 'filters-data-wrapper',
      'effect' => 'none',
      'method' => 'replace',
    ),
    '#name' => 'add_filter',
    '#submit' => array(), // Important, this skips normal submit
  );

  if (($fid = $context->getFid()) && $context->allowSave()) {
    $form['fid'] = array(
      '#type' => 'value',
      '#value' => $fid,
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#attributes' => array('class' => 'default-submit-action'),
      '#value' => t('Save'),
      '#submit' => array('assistant_api_filters_form_save_submit'),
    );

    if (drupal_get_destination()) {
      $form['cancel'] = array(
        '#type' => 'submit',
        '#value' => t('Cancel'),
        '#submit' => array(), // Skip all submit for cancel
      );
    }
  }

  // Set context for live preview block
  if ($context->allowPreview()) {

    $form['preview'] = array(
      '#type' => 'button',
      '#value' => t("Update preview"),
      '#ahah' => array(
        'event' => 'click',
        'path' => ahah_helper_path(array('live', 'preview')),
        'wrapper' => 'assistant-api-filter-form-live-preview',
        'effect' => 'fade',
        'method' => 'replace',
      ),
      '#name' => 'live_preview',
      '#submit' => array(), // Important, this skips normal submit
    );

    if (assistant_api_use_jquery_ui()) {
      drupal_add_js(drupal_get_path('module', 'assistant_api') . '/assistant_api.form.ui.js');
      $form['preview']['#value'] = t('Show preview');
    }

    $form['live'] = array(
      '#type' => 'fieldset',
      '#title' => t('Live preview ..'),
      '#collapsible' => FALSE,
      '#attributes' => array('class' => 'assistant-dialog-main'),
    );
    $form['live']['preview'] = array(
      '#prefix' => '<div id="assistant-api-filter-form-live-preview">',
      '#suffix' => '</div>',
      '#value' => '<span> </span>', // Display this element even if empty 
    );

    if ($do_preview) {
      $form['live']['preview']['#value'] = _assistant_api_filters_form_preview($form_state['storage']['data'], $context);
    }
  }

  return $form;
}

function _assistant_api_filters_validate(&$filters, AbstractAssistantContext $context) {
  $errors = array();

  assistant_api_filters_boostrap();

  foreach ($filters as $delta => &$element) {

    // Remove all 'remove' buttons from values
    array_pop($element);

    // Check for validation callback
    foreach ($element as $name => &$values) {
      $filter = AssistantFilterFactory::getFilterInstanceByName($name);
      if ($error = $filter->validate($context, $values)) {
        $element_name = 'data][' . $delta . '][' . $name . '][' . $error[0];
        $errors[$element_name] =  $error[1];
      }
    }
  }

  return $errors;
}

function assistant_api_filters_form_validate($form, &$form_state) {
  if (! empty($form_state['values']['data'])) {
    $errors = _assistant_api_filters_validate($form_state['values']['data'], $form_state['values']['context']);
    if (!empty($errors)) {
      foreach ($errors as $element_name => &$error) {
        form_set_error($element_name, $error);
      }
    }
  }
}

/**
 * Submit callback for filters form when form is for a node
 */
function assistant_api_filters_form_save_submit($form, &$form_state) {
  // Save new filter
  $filters = $form_state['values']['data'];
  if (isset($form_state['values']['fid'])) {
    $filters['fid'] = $form_state['values']['fid'];
  }
  assistant_api_filters_save($filters);

  drupal_set_message(t("Filters saved"));

  // Ensure ahah_helper stuff does not return some strange array
  $form_state['rebuild'] = FALSE;
  $form_state['redirect'] = isset($_GET['destination']) ? $_GET['destination'] : $_GET['q'];

  drupal_goto($form_state['redirect']);
}

/**
 * Helper for filter form.
 */
function _assistant_api_filters_add_subform(AbstractAssistantContext $context) {
  $form = array();
  $options = array();
  foreach (AssistantFilterFactory::getFilterNamesByContext($context) as $name) {
    $filter = AssistantFilterFactory::getFilterInstanceByName($name);
    $options[$name] = t($filter->getTitle());
  }
  $form['filter_name'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => NULL, 
  );
  return $form;
}

/**
 * Helper that build live preview result
 */
function _assistant_api_filters_form_preview(&$filters, AbstractAssistantContext $context) {
  $output = '';

  // Force our elements validation, do nothing if validation fails
  $errors = _assistant_api_filters_validate($filters, $context);

  if (!empty($errors)) {
    return t("Got validation errors");
  }

  // Set the context in preview mode
  $context->setPreview(TRUE);
  $nid_list = assistant_api_solr_filters_query($filters, $context);

  return theme('assistant_preview', $nid_list, $context);
}

/**
 * Get assistant filter subform.
 */
function _assistant_api_filters_form(AbstractAssistantContext $context, $name, $delta, &$values) {
  $filter = AssistantFilterFactory::getFilterInstanceByName($name);
  $filter->setAhahHelperPath(array('data', $delta, $name));
  return $filter->form($context, $values);
}

/**
 * Build filter form with user set filters for given field_name and node.
 * This will load user set filter data and build the form with those values. If
 * no values given, an empty form array is returned.
 * 
 * @param AbstractAssistantContext $context
 *   AbstractAssistantContext instance.
 * @param object &$settings
 *   Assistant feature settings object for node and field_name
 * 
 * @return array
 *   Valid Form API elements subset
 */
function _assistant_api_filters_subform(AbstractAssistantContext $context, &$filters) {
  $form = array();
  if (! empty($filters)) {
    foreach ($filters as $delta => &$data) {
      if (is_array($data)) {
        list($name, $values) = each($data);
        $form[] = array($name => _assistant_api_filters_form($context, $name, count($form), $values));
      }
    }
  }
  return $form;
}

/**
 * Theme filters building form.
 * 
 * TODO: temporary deactivated fuzzy search because it seems that SolR 1.4 has
 * some problems using it in DisMax.
 * 
 * @see theme_assistant_api_ahah_filter()
 */
function theme_assistant_api_filters_form_data($form) {
  $headers = array(t("Filter"), t("Settings"), /* t('Fuzzy'), */ t("Boost"), "");
  $rows = array();

  foreach (element_children($form) as $key) {
    // Render 'remove' button
    $row_remove = drupal_render($form[$key]['remove']);
    // Ugly weight element lookup and render
    $row_fuzzy = NULL;
    $row_boost = NULL;
    $row_title = NULL;
    $row_is_filter = FALSE;
    foreach ($form[$key] as &$filter_element) {;
      if (isset($filter_element['title'])) {
        $row_title = drupal_render($filter_element['title']);
      }
      if (isset($filter_element['boost'])) {
        unset($filter_element['boost']['#title']);
        $row_boost = drupal_render($filter_element['boost']);
      }
      if ($filter_element['filter_query']) {
        $row_is_filter = (bool) $filter_element['filter_query']['#value'];
      }
      /* if (isset($filter_element['fuzzy'])) {
        unset($filter_element['fuzzy']['#title']);
        $row_fuzzy = drupal_render($filter_element['fuzzy']);
      } */
    }
    /* if (empty($row_fuzzy)) {
      $row_fuzzy = '';
    } */
    if (empty($row_boost)) {
      $row_boost = '';
    }
    // Render all other elements
    $row_filter = $prefix . drupal_render($form[$key]) . $suffix;
    $row = array(
      'data' => array($row_title, $row_filter, /* $row_fuzzy, */ $row_boost, $row_remove),
      'class' => ($row_is_filter ? 'filter-query' : 'query'),
    );
    $rows[] = $row;
  }

  return theme('table', $headers, $rows) . drupal_render($form);
}